--Written by Dave Wortley
--www.facebook.com/maxmadeasy
--davewortley.wordpress.com
--2019
--Use this to allow unique aspect ratios of camera projections in one scene.

macroScript DW_MaterialOverrideExcluder 
 	category:"DW Tools" 
 	buttontext:"Camera Map Ratio Fixer" 
 	tooltip:"Camera Map Texture Ratio Fixer"
 (

struct cameraMapRatio
(
	fn getfilefromIFL iflFile =
	(
		local file = openFile iflFile
		local filepath = getFilenamePath iflFile
		filename = readLine file
		close file 
		
		if findString filename "\\" == undefined do
			filename = filepath + filename
		filename
	),

	fn collectMaps mapTarget mapID =
	(
		maps = #()
		join maps (getclassinstances bitmaptexture target:mapTarget)
		join maps (getclassinstances vrayhdri target:mapTarget)
		
		(for m in maps where m.coords.mapChannel == mapID collect m)
	),

	fn setMapRatio cam matchingMaps =
	(
		--Get image aspect
		fName = Case classof matchingMaps[1] of
		(
			bitmaptexture: matchingMaps[1] .filename
			vrayhdri: matchingMaps[1] .hdrifilename
		)
		
		if fName != undefined and doesFileExist fName and getfilenametype fName == ".ifl" do
			fName = cameraMapRatio.getfilefromIFL fName
		
		if fName != undefined and doesFileExist fName do
		(
			img = (dotNetClass "System.Drawing.Image").FromFile fName
			imgSize = [img.Height, img.Width]
			imageAspect = 1.0 * imgSize[2] / imgSize[1]
			
			--Get render aspect
			renderAspect = 1.0 * renderWidth / renderHeight
			
			--scale camera
			cam.scale.x = 1.0
			cam.scale.y = renderAspect/ imageAspect
			cam.scale.z = 1.0
			
			--Update Camera
			
			
		)
	),

	fn setCameraMapAspect cam =
	(
		if superclassof cam == camera do
		(
			cameraMapMods = (for o in (refs.dependents cam) where classof o == CameraMap or classof o == SpaceCameraMap collect o)
			
			cameraMapPerPixelTextures  = (for o in (refs.dependents cam) where classof o == CameraMapTexture collect o)
			
			if cameraMapMods.count != 0 do
			(
				cameraMapMod = cameraMapMods[1]
				mapID = cameraMapMod.channel
				
				cameraMapObj = (refs.dependentNodes cam)[1]
				
				if cameraMapObj.material != undefined do
				(
					matchingMaps = cameraMapRatio.collectMaps cameraMapObj.material mapID
					
					if matchingMaps.count != 0 do
					(
						cameraMapRatio.setMapRatio cam matchingMaps
						cameraMapMod.cameraNode = cameraMapMod.cameraNode
					)
				)
			)
			
			if cameraMapperPixelTextures.count != 0 do
			(
				cameraMapperPixelTexture = cameraMapperPixelTextures[1]
				mapID = cameraMapperPixelTexture.MapChannel
				
				matchingMaps = cameraMapRatio.collectMaps cameraMapperPixelTexture.texture mapID
					
				if matchingMaps.count != 0 do
				(
					cameraMapRatio.setMapRatio cam matchingMaps
				)
				
				cameraMapperPixelTexture.camera = cameraMapperPixelTexture.camera
			)
		)
	),

	fn updateAllCameraMaps =
	(
		for o in cameras where superclassof o == camera do cameraMapRatio.setCameraMapAspect o
	),

	fn updateSelectedCameraMaps =
	(
		for o in selection where superclassof o == camera do cameraMapRatio.setCameraMapAspect o
	),
	
	RL_CameraMapRatio = rollout RL_CameraMapRatio "Camera Map Ratio" width:220
	(
		group "Adjust CameraMap Cameras"
		(
			button btn_correctAspect "Correct Aspect on Selected" width:200 tooltip:"Correct the aspect ratio (scale cameras in Y) on the selected camera."
			button btn_correctAllAspect "Correct All Aspect Ratios" width:200 tooltip:"Correct the aspect ratio (scale cameras in Y) on all the projection cameras in the scene"
		)
		group "About"
		(
			label lbl_dw "Written by Dave Wortley"
			hyperlink hpl_fb "facebook.com/maxmadeeasy" align:#center address:"https://www.facebook.com/maxmadeeasy"
		)
		
		on btn_correctAspect pressed do
		(
			cameraMapRatio.updateSelectedCameraMaps()
		)
		on btn_correctAllAspect pressed do
		(
			cameraMapRatio.updateAllCameraMaps()
		)
	),
	
	fn destroy =
	(
		try(destroyDialog ::cameraMapRatioStruct.RL_CameraMapRatio)catch()
	),
	
	fn showUI  =
	(
		destroy()
		createDialog RL_CameraMapRatio
	),
	on create do
	(
		destroy()
	)
)
cameraMapRatioStruct = cameraMapRatio()

cameraMapRatioStruct.showUI()

)